from setuptools import setup

setup(
    name='general_util',
    version='0.0.18.1',
    packages=['general_util', 'general_util.log', 'general_util.log.proto'],
    install_requires=[
        'aiohttp>=3.10.3',
        'fastapi>=0.83.0',
        'grpcio>=1.65.4',
        'grpcio-tools>=1.65.4',
        'PyYAML>=6.0.2',
        'requests>=2.32.3',
    ],
    author='wilson',
    author_email='wwilson008@gmail.com',
    description='general operator for fastapi write sql and redis',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/littlebluewhite/node_object_module',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ],
    python_requires='>=3.8',
)




import time
import grpc
import requests
from fastapi import Request
from fastapi.responses import JSONResponse
from .data import url_module_mapping, status_code_rules
from .system_log_schema import Log
from .proto import system_log_pb2, system_log_pb2_grpc


class DealSystemLog:
    def __init__(self, request: Request, response: JSONResponse, url_mapping:dict=None, code_rules=None):
        self.request = request
        self.response = response
        self.url_mapping = url_module_mapping
        self.code_rules = status_code_rules
        if url_mapping is not None:
            self.url_mapping = url_mapping
        if code_rules is not None:
            self.code_rules = code_rules

    async def deal(self, system_log_g_server, timeout=5.0):
        log = self.__create_system_log()
        print("write log: ", self.__write_log_g(log, system_log_g_server, timeout=timeout))

    async def deal_r(self, system_log_r_server, timeout=5.0):
        log = self.__create_system_log()
        print("write log: ", self.__write_log_r(log, system_log_r_server, timeout=timeout))


    def __create_system_log(self):
        response_headers: dict = dict(self.response.headers)
        request_headers: dict  = dict(self.request.headers)
        module, submodule, item = self.__get_module_submodule_item()
        response_headers["check_header"] = "check_value"
        self.response.init_headers(response_headers)
        message, message_code = self.__get_message(response_headers)
        t = time.time()
        # get account
        account = request_headers.get("account", "")
        state_account = getattr(self.request.state, "account", None)
        if state_account:
            account = state_account
        print("request_headers: ", request_headers)
        print("x-forwarded-for: ", request_headers.get("x-forwarded-for", "None"))
        return Log(
            timestamp= t,
            module=module,
            submodule=submodule,
            item=item,
            method=self.request.method,
            status_code=f"{self.response.status_code}",
            message_code=message_code,
            message=message,
            response_size=response_headers.get("content-length", ""),
            account=account,
            ip=request_headers.get("x-forwarded-for", self.request.client.host),
            api_url=self.request.url.path,
            query_params=self.request.url.query,
            web_path=request_headers.get("web-path", "")
        )

    def __get_module_submodule_item(self):
        if self.url_mapping.get(self.request.url.path, None) is not None:
            return self.url_mapping.get(self.request.url.path)
        else:
            url_path = self.request.url.path.split("/")
            if len(url_path) < 5:
                return "UNKNOWN", "UNKNOWN", "UNKNOWN"
            else:
                return url_path[2], url_path[3], url_path[4]

    def __get_message(self, headers):
        status_code = str(self.response.status_code)
        message = ""
        message_code = ""
        if status_code in self.code_rules:
            message = str(self.code_rules[status_code]["message"])
            message_code = str(self.code_rules[status_code]["message_code"])
        # 以headers為主
        h_message = headers.get("message", None)
        if h_message is not None:
            message = h_message
        h_message_code = headers.get("message_code", None)
        if h_message_code is not None:
            message_code = h_message_code
        return message, message_code

    @staticmethod
    def __write_log_g(log, system_log_g_server, timeout=5.0):
        with grpc.insecure_channel(f"{system_log_g_server}") as channel:
            stub = system_log_pb2_grpc.SystemLogServiceStub(channel)
            request = system_log_pb2.LogRequest(
                timestamp=log.timestamp,
                module=log.module,
                submodule=log.submodule,
                item=log.item,
                method=log.method,
                status_code=log.status_code,
                message_code=log.message_code,
                message=log.message,
                response_size=log.response_size,
                account=log.account,
                ip=log.ip,
                api_url=log.api_url,
                query_params=log.query_params,
                web_path=log.web_path
            )
            return stub.WriteLog(request, timeout=timeout)

    @staticmethod
    def __write_log_r(log, system_log_r_server, timeout=5.0):
        body = {
            "timestamp": log.timestamp,
            "module": log.module,
            "submodule": log.submodule,
            "item": log.item,
            "method": log.method,
            "status_code": log.status_code,
            "message_code": log.message_code,
            "message": log.message,
            "response_size": log.response_size,
            "account": log.account,
            "ip": log.ip,
            "api_url": log.api_url,
            "query_params": log.query_params,
            "web_path": log.web_path
        }
        url = f'{system_log_r_server}/api/log/'
        headers = {
            'accept': 'application/json',
            'Content-Type': 'application/json'
        }
        response = requests.post(url, json=body, headers=headers, timeout=timeout)
        return response




import os
import yaml
import configparser


class ToDict:

    @classmethod
    def to_dict(cls):
        return cls.data

    @classmethod
    def set_data(cls, data):
        cls.data = data


class ConfigManager:

    @classmethod
    def __register_attribute(cls, cur_cls, data: dict = None):
        if data is None:
            data = dict()
        for k, v in data.items():
            if isinstance(v, dict):
                class _(ToDict):
                    pass

                _.set_data(v)
                cls.__register_attribute(_, v)
                setattr(cur_cls, k, _)
            else:
                setattr(cur_cls, k, v)
        # cls = cur_cls if cls is cur_cls else ...

    @classmethod
    def read_yaml(cls, yaml_file: str):
        # Load YAML into Python dictionary
        with open(yaml_file, 'r') as file:
            yaml_dict = yaml.safe_load(file)
            cls.__override_with_env_variables(yaml_dict)
            cls.__register_attribute(cur_cls=cls, data=yaml_dict)

    @classmethod
    def read_ini(cls, ini_file: str):
        config = configparser.ConfigParser()
        config.read(ini_file)
        cls.__override_with_env_variables(config._sections)
        cls.__register_attribute(cur_cls=cls, data=config._sections)

    @staticmethod
    def convert_type(value, original_type):
        """Convert the value to the original type."""
        try:
            if original_type is bool:
                return value.lower() in {"true", "1", "t", "y", "yes", "True", "T"}
            elif original_type is list:
                return [v.strip() for v in value.split(',')]
            elif original_type is type(None):
                return value
            else:
                return original_type(value)
        except ValueError:
            return value

    @classmethod
    def __override_with_env_variables(cls, config, prefix=''):
        """Recursively override configuration with environment variables if they exist."""
        if isinstance(config, dict):
            for key, value in config.items():
                env_key = f"{prefix}{key}".upper()
                if isinstance(value, dict):
                    cls.__override_with_env_variables(value, prefix=env_key + '_')
                else:
                    env_value = os.environ.get(env_key)
                    if env_value is not None:
                        config[key] = cls.convert_type(env_value, type(value))
        return config




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: function/log/proto/system_log.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#function/log/proto/system_log.proto\"\x89\x02\n\nLogRequest\x12\x11\n\ttimestamp\x18\x01 \x01(\x01\x12\x0e\n\x06module\x18\x02 \x01(\t\x12\x11\n\tsubmodule\x18\x03 \x01(\t\x12\x0c\n\x04item\x18\x04 \x01(\t\x12\x0e\n\x06method\x18\x05 \x01(\t\x12\x13\n\x0bstatus_code\x18\x06 \x01(\t\x12\x14\n\x0cmessage_code\x18\x07 \x01(\t\x12\x0f\n\x07message\x18\x08 \x01(\t\x12\x15\n\rresponse_size\x18\t \x01(\t\x12\x0f\n\x07\x61\x63\x63ount\x18\n \x01(\t\x12\n\n\x02ip\x18\x0b \x01(\t\x12\x0f\n\x07\x61pi_url\x18\x0c \x01(\t\x12\x14\n\x0cquery_params\x18\r \x01(\t\x12\x10\n\x08web_path\x18\x0e \x01(\t\"\x1e\n\x0bLogResponse\x12\x0f\n\x07message\x18\x01 \x01(\t29\n\x10SystemLogService\x12%\n\x08WriteLog\x12\x0b.LogRequest\x1a\x0c.LogResponseb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'function.log.proto.system_log_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_LOGREQUEST']._serialized_start=40
  _globals['_LOGREQUEST']._serialized_end=305
  _globals['_LOGRESPONSE']._serialized_start=307
  _globals['_LOGRESPONSE']._serialized_end=337
  _globals['_SYSTEMLOGSERVICE']._serialized_start=339
  _globals['_SYSTEMLOGSERVICE']._serialized_end=396
# @@protoc_insertion_point(module_scope)



import asyncio
import aiohttp
import requests


async def fetch(urls: list[str], headers: dict = None, timeout: int = 5):
    timeout_setting = aiohttp.ClientTimeout(total=timeout)
    async with aiohttp.ClientSession(timeout=timeout_setting) as session:
        tasks = [fetch_data(session, url, headers) for url in urls]
        return await asyncio.gather(*tasks)


async def fetch_data(session, url, headers):
    try:
        async with session.get(url, headers=headers) as response:
            return await response.json()
    except asyncio.TimeoutError:
        print("error", f"Timeout occurred for {url}")
        raise asyncio.TimeoutError
    except Exception as e:
        print("error", f"An error occurred while fetching {url}: {e}")
        raise e


def sync_fetch(urls: list[str], headers: dict = None, timeout: int = 5):
    result = []
    try:
        for url in urls:
            response = requests.get(url, headers=headers, timeout=timeout)  # Pass timeout to the request
            response.raise_for_status()  # Check for HTTP errors
            result.append(response.json())  # Return the response content
    except requests.Timeout:
        print("Timeout occurred")
        raise requests.Timeout
    except requests.RequestException as e:
        print("An error occurred:", e)
        raise e
    return result


from pydantic import BaseModel


class Log(BaseModel):
    timestamp: float
    module: str
    submodule: str
    item: str
    method: str
    status_code: str
    message_code: str = ""
    message: str = ""
    response_size: str
    account: str = ""
    ip: str
    api_url: str
    query_params: str = ""
    web_path: str = ""



url_module_mapping = {
    "/docs": ("custom", "document", "docs"),
    "/openapi.json": ("custom", "document", "docs2")
}

status_code_rules = {
    "401": {"message": "Unauthorized", "message_code": "401"},
    "403": {"message": "Forbidden", "message_code": "403"},
    "422": {"message": "Unprocessable Entity", "message_code": "422"},
}
